QuestHelper_File["help.lua"] = "0.59"

function QuestHelper:scaleString(val)
  return self:HighlightText(math.floor(val*100+0.5).."%")
end

function QuestHelper:genericSetScale(varname, name, mn, mx, input, onchange, ...)
  if input == "" then
    self:TextOut(string.format("Текущий параметр %s установлен на %s.", name, self:scaleString(QuestHelper_Pref[varname])))
  else
    local scale = tonumber(input)
    
    if not scale then
      local x = string.match(input or "", "^%s*([%d%.]+)%s*%%%s*$")
      scale = tonumber(x) or 0
      if not scale then
        self:TextOut("Невозможно понять вводимые значения.")
        return
      end
      scale = scale * 0.01
    end
    
    if scale < mn then
      self:TextOut(string.format("Невозможно сделать параметр меньше %s.", self:scaleString(mn)))
    elseif scale > mx then
      self:TextOut(string.format("Невозможно сделать параметр больше %s.", self:scaleString(mx)))
    else
      QuestHelper_Pref[varname] = scale
      self:TextOut(string.format("Параметр %s установлен на %s.", name, self:scaleString(scale)))
      if onchange then
        onchange(...)
      end
    end
  end
end

function QuestHelper:TrackerScale(scale)
  QuestHelper:genericSetScale("track_scale", "размера списка заданий", .5, 2, scale,
                              function() QuestHelper.tracker:SetScale(QuestHelper_Pref.track_scale) end)
end

function QuestHelper:SetLocale(loc)
  if not loc or loc == "" then
    self:TextOut(QHText("LOCALE_LIST_BEGIN"))
    for loc, tbl in pairs(QuestHelper_Translations) do
      self:TextOut(string.format("  %s%s %s", self:HighlightText(loc),
                                              loc == QuestHelper_Pref.locale and " *" or "  ",
                                              tbl.LOCALE_NAME or "???"))
    end
  else
    for l, tbl in pairs(QuestHelper_Translations) do
      if string.find(string.lower(l), "^"..string.lower(loc)) or
         string.find(string.lower(tbl.LOCALE_NAME or ""), "^"..string.lower(loc)) then
        QuestHelper_Pref.locale = l
        QHFormatSetLocale(l)
        self:SetLocaleFonts()
        self:TextOut(QHFormat("LOCALE_CHANGED", l))
        return
      end
    end
    
    self:TextOut(QHFormat("LOCALE_UNKNOWN", tostring(loc) or "UNKNOWN"))
  end
end

function QuestHelper:ToggleHide()
  local current_objective = self.minimap_dodad.objective
  
  QuestHelper_Pref.hide = not QuestHelper_Pref.hide
  
  -- Desaturate the button texture if QuestHelper is disabled.
  if self.MapButton then
    -- This should always be true, but just in case...
    self.MapButton:GetNormalTexture():SetDesaturated(QuestHelper_Pref.hide)
  end
  
  if QuestHelper_Pref.hide then
    if QuestHelper_Pref.track then
      self:HideTracker()
    end
    
    self.map_overlay:Hide()
    self.minimap_dodad:SetObjective(nil)
    self.minimap_dodad.objective = current_objective
    self:TextOut("QuestHelper теперь |cffff0000отключен|r.")
  else
    if QuestHelper_Pref.track then
      self:ShowTracker()
    end
    
    self.map_overlay:Show()
    self.minimap_dodad.objective = nil
    self.minimap_dodad:SetObjective(current_objective)
    self:TextOut("QuestHelper теперь |cff00ff00включен|r.")
    -- WoW Will lockup inside ForceRouteUpdate, and so the UPDATING_ROUTE message won't appear until afterwards, making
    -- this message kind of redundant.
    -- self:TextOut(QHText("UPDATING_ROUTE"))
    self:ForceRouteUpdate(4)        -- Let the corutine do some overtime...
  end
end

function QuestHelper:ToggleShare()
  QuestHelper_Pref.share = not QuestHelper_Pref.share
  if QuestHelper_Pref.share then
    if QuestHelper_Pref.solo then
      self:TextOut("Обмен целями будет |cff00ff00включен|r, когда вы отключите игнорирование группы.")
    else
      self:TextOut("Обмен целями теперь |cff00ff00включен|r.")
      self:EnableSharing()
    end
  else
    self:TextOut("Обмен целями теперь |cffff0000отключен|r.")
    if QuestHelper_Pref.solo then
      self:TextOut("Обмен целями не может быть |cff00ff00включен|r, пока вы не отключите игнорирование группы.")
    else
      self:DisableSharing()
    end
  end
end

function QuestHelper:ToggleFlightTimes()
  QuestHelper_Pref.flight_time = not QuestHelper_Pref.flight_time
  if QuestHelper_Pref.flight_time then
    self:TextOut("Таймер полёта теперь |cff00ff00включен|r.")
  else
    self:TextOut("Таймер полёта теперь |cffff0000отключен|r.")
  end
end

function QuestHelper:ToggleTrack()
  QuestHelper_Pref.track = not QuestHelper_Pref.track
  if QuestHelper_Pref.track then
    self:ShowTracker()
    self:TextOut("Отслеживание заданий теперь |cff00ff00включено|r.")
  else
    self:HideTracker()
    self:TextOut("Отслеживание заданий теперь |cffff0000отключено|r.")
  end
end

function QuestHelper:ToggleTrackLevel()
  QuestHelper_Pref.track_level = not QuestHelper_Pref.track_level
  if QuestHelper_Pref.track_level then
    self:TextOut("Отображение уровней заданий в отслеживании заданий теперь |cff00ff00включено|r.")
  else
    self:TextOut("Отображение уровней заданий в отслеживании заданий теперь |cffff0000отключено|r.")
  end
  self.tracker:reset()
end

function QuestHelper:ToggleTrackQColour()
  QuestHelper_Pref.track_qcolour = not QuestHelper_Pref.track_qcolour
  if QuestHelper_Pref.track_qcolour then
    self:TextOut("Окраска заданий по сложности в отслеживании заданий теперь |cff00ff00включена|r.")
  else
    self:TextOut("Окраска заданий по сложности в отслеживании заданий теперь |cffff0000отключена|r.")
  end
  self.tracker:reset()
end

function QuestHelper:ToggleTrackOColour()
  QuestHelper_Pref.track_ocolour = not QuestHelper_Pref.track_ocolour
  if QuestHelper_Pref.track_ocolour then
    self:TextOut("Индикация прогресса цветом в отслеживании заданий теперь |cff00ff00включена|r.")
  else
    self:TextOut("Индикация прогресса цветом в отслеживании заданий теперь |cffff0000отключена|r.")
  end
  self.tracker:reset()
end

function QuestHelper:ToggleTooltip()
  QuestHelper_Pref.tooltip = not QuestHelper_Pref.tooltip
  if QuestHelper_Pref.tooltip then
    self:TextOut("Подсказка цели теперь |cff00ff00включена|r.")
  else
    self:TextOut("Подсказка цели теперь |cffff0000отключена|r.")
  end
end

function QuestHelper:Purgewarning()
  QuestHelper:TextOut("Я уверен, что это печальная утрата и был бы признателен, если бы вы отправили мне свои сохранённые данные перед тем, как сделать это.")
  QuestHelper:TextOut("Введите "..self:HighlightText("/qh nag verbose")..", чтобы проверить уничтожаемое на что-нибудь важное.")
  QuestHelper:TextOut("Введите "..self:HighlightText("/qh submit")..", чтобы посмотреть инструкции по отправке данных.")
  QuestHelper:TextOut("Смотрите раздел "..self:HighlightText("How You Can Help").." на сайте, для инструкций.")
end

function QuestHelper:Purge(code, force, noreload)
  if code == self.purge_code or force then
    QuestHelper_Quests = nil
    QuestHelper_Objectives = nil
    QuestHelper_FlightInstructors = nil
    QuestHelper_FlightRoutes = nil
    QuestHelper_Locale = nil
    QuestHelper_UID = nil
    QuestHelper_Version = nil
    QuestHelper_SaveVersion = nil
    
    QuestHelper_SaveDate = nil
    QuestHelper_SeenRealms = nil
    
    if not noreload then ReloadUI() end
  else
    if not self.purge_code then self.purge_code = self:CreateUID(8) end
    QuestHelper:TextOut("ЭТА КОМАНДА УДАЛИТ ВСЕ ВАШИ СОБРАННЫЕ ДАННЫЕ")
    QuestHelper:Purgewarning()
    QuestHelper:TextOut("Если вы уверены, что хотите сделать это, то введите: "..self:HighlightText("/qh purge "..self.purge_code))
  end
end

function QuestHelper:HardReset(code)
  if code == self.purge_code then
    QuestHelper_Pref = nil
    QuestHelper_ErrorList = nil -- BIZAM
    QuestHelper_KnownFlightRoutes = nil
    QuestHelper_Home = nil
    QuestHelper_CharVersion = nil
    self:Purge(nil, true)
  else
    if not self.purge_code then self.purge_code = self:CreateUID(8) end
    QuestHelper:TextOut("ЭТА КОМАНДА УДАЛИТ ВСЕ ВАШИ СОБРАННЫЕ ДАННЫЕ И СБРОСИТЬ ВСЕ ВАШИ НАСТРОЙКИ")
    QuestHelper:Purgewarning()
    QuestHelper:TextOut("Если вы уверены, что хотите сделать это, то введите: "..self:HighlightText("/qh hardreset "..self.purge_code))
  end
end

function QuestHelper:ToggleSolo()
  QuestHelper_Pref.solo = not QuestHelper_Pref.solo
  if QuestHelper_Pref.solo then
    if QuestHelper_Pref.share then
      self:DisableSharing()
      self:TextOut("Обмен целями временно |cffff0000отключен|r.")
    end
    self:TextOut("Игнорирование группы теперь |cff00ff00включено|r.")
  else
    self:TextOut("Игнорирование группы теперь |cffff0000отключено|r.")
    if QuestHelper_Pref.share then
      self:EnableSharing()
      self:TextOut("Обмен целями был пере|cff00ff00подключен|r.")
    end
  end
end

function QuestHelper:ToggleComm()
  if QuestHelper_Pref.comm then
    QuestHelper_Pref.comm = false
    self:TextOut("Отображение коммуникации теперь |cffff0000отключено|r.")
  else
    QuestHelper_Pref.comm = true
    self:TextOut("Отображение коммуникации теперь |cff00ff00включено|r.")
  end
end

function QuestHelper:ToggleAnts()
  if QuestHelper_Pref.show_ants then
    QuestHelper_Pref.show_ants = false
    self:TextOut("Оптимальный путь теперь |cffff0000отключен|r.")
  else
    QuestHelper_Pref.show_ants = true
    self:TextOut("Оптимальный путь теперь |cff00ff00включен|r.")
  end
end

function QuestHelper:LevelOffset(offset)
  local level = tonumber(offset)
  if level then
    if level > 0 then
      self:TextOut("Отображение заданий также на "..self:HighlightText(level).." уров"..(level==1 and "ень " or level==5 and "ней " or "ня ")..self:HighlightText("выше").." вашего.")
    elseif level < 0 then
      self:TextOut("Отображение заданий только на "..self:HighlightText(-level).." уров"..(level==-1 and "ень " or level==-5 and "ней " or "ня ")..self:HighlightText("ниже").." вашего.")
    else
      self:TextOut("Отображение заданий "..self:HighlightText("выше и ниже").." вашего текущего уровня.")
    end
    
    if not QuestHelper_Pref.filter_level then
      self:TextOut("Примечание: Вы не увидите эффекта, пока не включите фильтр по уровню заданий.")
    end
    
    if QuestHelper_Pref.level ~= level then
      QuestHelper_Pref.level = level
      self.defered_quest_scan = true
    end
  elseif offset == "" then
    if QuestHelper_Pref.level <= 0 then
      self:TextOut("Установлено смещение по уровню на "..self:HighlightText(QuestHelper_Pref.level)..".")
    else
      self:TextOut("Установлено смещение по уровню на "..self:HighlightText("+"..QuestHelper_Pref.level)..".")
    end
    
    if self.party_levels then for n, l in ipairs(self.party_levels) do
      self:TextOut("Для заданий на "..self:HighlightText(n).." игрок"..(n==1 and "а" or "ов")..", вы или ваша группа эффективно можете справится с заданием "..self:HighlightText(string.format("%.0f", l)).." уровня.")
    end end
    
    if QuestHelper_Pref.solo then
      self:TextOut("Эффективность, зависящая от игроков в группе, не учитывается, так как у вас включено игнорирование групп.")
    end
  else
    self:TextOut("Ожидается смещение по уровню.")
  end
end

function QuestHelper:Filter(input)
  input = string.upper(input)
  if input == "ZONE" then
    QuestHelper_Pref.filter_zone = not QuestHelper_Pref.filter_zone
    self:TextOut("Фильтр "..self:HighlightText("по локациям").." "..self:HighlightText(QuestHelper_Pref.filter_zone and "|cff00ff00включен|r" or "|cffff0000отключен|r")..".")
  elseif input == "DONE" then
    QuestHelper_Pref.filter_done = not QuestHelper_Pref.filter_done
    self:TextOut("Фильтр "..self:HighlightText("по завершённым заданиям").." "..self:HighlightText(QuestHelper_Pref.filter_done and "|cff00ff00включен|" or "|cffff0000отключен|r")..".")
  elseif input == "LEVEL" then
    QuestHelper_Pref.filter_level = not QuestHelper_Pref.filter_level
    self:TextOut("Фильтр "..self:HighlightText("по уровню").." "..self:HighlightText(QuestHelper_Pref.filter_level and "|cff00ff00включен|r" or "|cffff0000отключен|r")..".")
  elseif input == "BLOCKED" or input == "BLOCK" then
    QuestHelper_Pref.filter_blocked = not QuestHelper_Pref.filter_blocked
    self:TextOut("Фильтр "..self:HighlightText("по заблокированным заданиям").." "..self:HighlightText(QuestHelper_Pref.filter_blocked and "|cff00ff00включен|r" or "|cffff0000отключен|r")..".")
  elseif input == "WATCHED" or input == "WATCH" then
    QuestHelper_Pref.filter_watched = not QuestHelper_Pref.filter_watched
    self:TextOut("Фильтр "..self:HighlightText("по отслеживаемым заданиям").." "..self:HighlightText(QuestHelper_Pref.filter_watched and "|cff00ff00включен|r" or "|cffff0000отключен|r")..".")
  elseif input == "" then
    self:TextOut("Фильтр "..self:HighlightText("по локациям").." "..self:HighlightText(QuestHelper_Pref.filter_zone and "|cff00ff00включен|r" or "|cffff0000отключен|r"))
    self:TextOut("Фильтр "..self:HighlightText("по уровню").." "..self:HighlightText(QuestHelper_Pref.filter_level and "|cff00ff00включен|r" or "|cffff0000отключен|r"))
    self:TextOut("Фильтр "..self:HighlightText("по завершённым заданиям").." "..self:HighlightText(QuestHelper_Pref.filter_done and "|cff00ff00включен|r" or "|cffff0000отключен|r"))
    self:TextOut("Фильтр "..self:HighlightText("по заблокированным заданиям").." "..self:HighlightText(QuestHelper_Pref.filter_blocked and "|cff00ff00включен|r" or "|cffff0000отключен|r"))
    self:TextOut("Фильтр "..self:HighlightText("по отслеживаемым заданиям").." "..self:HighlightText(QuestHelper_Pref.filter_watched and "|cff00ff00включен|r" or "|cffff0000отключен|r"))
  else
    self:TextOut("Не знаю такого фильтра, пробуйте фильтр по локациям ("..self:HighlightText("/qh filter zone").."), по завершённым заданиям ("..self:HighlightText("/qh filter done").."), по уровню ("..self:HighlightText("/qh filter level").."), по отслеживаемым заданиям ("..self:HighlightText("/qh filter watch")..") или по заблокированным заданиям ("..self:HighlightText("/qh filter block")..").")
  end
end

function QuestHelper:ToggleCartWP()
  QuestHelper_Pref.cart_wp = not QuestHelper_Pref.cart_wp
  if QuestHelper_Pref.cart_wp then
    self:EnableCartographer()
    if Cartographer_Waypoints then
      if Waypoint and Waypoint.prototype then
        self:TextOut("Отображение задач с помощью "..self:HightlightText("Cartographer Waypoints").." |cffff0000невозможно|r, так как другой мод мешает ему делать это.")
      else
        self:TextOut("Отображение задач с помощью "..self:HighlightText("Cartographer Waypoints").." теперь |cff00ff00включено|r.")
      end
    else
      self:TextOut("Отображение задач с помощью "..self:HighlightText("Cartographer Waypoints").." |cffff0000невозможно|r, так как не найдено самого аддона.")
    end
  else
    self:DisableCartographer()
    self:TextOut("Отображение задач с помощью "..self:HighlightText("Cartographer Waypoints").." теперь |cffff0000отключено|r.")
  end
end

function QuestHelper:ToggleTomTomWP()
  QuestHelper_Pref.tomtom_wp = not QuestHelper_Pref.tomtom_wp
  if QuestHelper_Pref.tomtom_wp then
    self:EnableTomTom()
    if TomTom then
      self:TextOut("Отображение задач с помощью "..self:HighlightText("TomTom").." теперь |cff00ff00включено|r.")
    else
      self:TextOut("Отображение задач с помощью "..self:HighlightText("TomTom").." |cffff0000невозможно|r, так как не найдено самого аддона.")
    end
  else
    self:DisableTomTom()
    self:TextOut("Отображение задач с помощью "..self:HighlightText("TomTom").." теперь |cffff0000отключено|r.")
  end
end

function QuestHelper:WantPathingReset()
  self:TextOut("Сброс всех направлений.")
  self.defered_graph_reset = true
end

function QuestHelper:PrintVersion()
  self:TextOut("Версия: "..self:HighlightText(GetAddOnMetadata("QuestHelper", "Version") or "Unknown"))
end

local function RecycleStatusString(fmt, usedcount, freetable, usedtable)
  local freetablecount = QuestHelper:TableSize(freetable)
  if usedtable then
    local usedtablecount = QuestHelper:TableSize(usedtable)
    return string.format(fmt, QuestHelper:ProgressString(string.format("%d/%d", usedtablecount, usedtablecount+freetablecount), ((usedtablecount+freetablecount == 0) and 1) or (1-usedtablecount/(usedtablecount+freetablecount)))) .. string.format(" (%d \"утекло\")", usedcount - usedtablecount)
  else
    return string.format(fmt, QuestHelper:ProgressString(string.format("%d/%d", usedcount, usedcount+freetablecount), ((usedcount+freetablecount == 0) and 1) or (1-usedcount/(usedcount+freetablecount))))
  end
end

function QuestHelper:RecycleInfo(cmd)
  self:TextOut(RecycleStatusString("Используется %s lua-таблиц.", self.used_tables, self.free_tables, self.recycle_tabletyping))
  self:TextOut(RecycleStatusString("Используется %s объектов текстур.", self.used_textures, self.free_textures))
  self:TextOut(RecycleStatusString("Используется %s объектов шрифта.", self.used_text, self.free_text))
  self:TextOut(RecycleStatusString("Используется %s объектов рамки.", self.used_frames, self.free_frames))
  
  if cmd and string.find(cmd, "verbose") then
    self:DumpTableTypeFrequencies()
  end
  
  if cmd and string.find(cmd, "cache") then
    self:DumpCacheData()
  end
  
  UpdateAddOnMemoryUsage()
  self:TextOut(string.format("QuestHelper использует %dкб оперативной памяти (%s/%s/%s/%s).", GetAddOnMemoryUsage("QuestHelper"), QuestHelper_local_version, QuestHelper_toc_version, GetBuildInfo(), GetLocale()))
end

function QuestHelper:ToggleMapButton()
  QuestHelper_Pref.map_button = not QuestHelper_Pref.map_button

  if QuestHelper_Pref.map_button then
    QuestHelper:InitMapButton()
    self:TextOut("Кнопка на карте теперь |cff00ff00включена|r.")
  else
    QuestHelper:HideMapButton()
    self:TextOut("Кнопка на карте теперь |cffff0000отключена|r. Введите '/qh button', чтобы вновь включить её.")
  end
end

function QuestHelper:ChangeLog()
  self:ShowText(QuestHelper_ChangeLog, string.format("QuestHelper %s, история изменений", QuestHelper_Version))
end

function QuestHelper:Submit()
  self:ShowText([[
|TInterface\AddOns\QuestHelper\Art\Upload.tga:100:300|t
Ваши данные не могут быть отправлены автоматически, так как аддонам запрещено взаимодействовать с чем-либо, кроме World of Warcraft.

Для этого требуется создание какой-то сторонней программы, но это может плохо кончится, так как кто-нибудь точно захочет сфальсифицировать её и использовать в своих корыстных целях.

World of Warcraft хранит данные QuestHelper'а в файле под названием |cff40bbffQuestHelper.lua|r.

Чтобы найти этот файл, сначала зайдите в папку, куда установлен World of Warcraft. В Windows по умолчанию это |cff40bbffC:\Program Files\World of Warcraft|r, и для Mac это должно быть |cff40bbff/Applications/World of Warcraft|r.

Если вы используете Windows Vista, система может запретить изменения в папке Program Files и перенаправить сохранённые данные Warcraft'а в  |cff40bbffC:\Users\|cffff8000USER|cff40bbff\AppData\Local\VirtualStore\Program Files\World of Warcraft|r.

В данной папке требуемый вам файл находится по пути |cff40bbffWTF/Account/|cffff8000ACCOUNT|cff40bbff/SavedVariables|r, замените название папки ACCOUNT на имя своей учётной записи и вы легко найдёте |cff40bbffQuestHelper.lua|r.

Также, существует и другие папки и файлы с аналогичными именами в папке с названиями серверов, где хранятся данные ваших персонажей, |cffffff00но они не нужны|r. Там содержится информация только для вашего персонажа, типа изученных точек перелёта, о которых итак всё известно.

После того, как вы обнаружите файл |cff40bbffQuestHelper.lua|r, отправьте его на почту: |cff40bbffqhaddon@gmail.com|r
]], "Как отправить ваши данные")
end

function QuestHelper:ShowError()
  QuestHelper_ErrorCatcher_ReportError()
end

local commands

function QuestHelper:Help(argument)
  local text = ""
  local argument = argument and argument:upper() or ""
  
  for i, data in ipairs(commands) do
    if data[1]:find(argument, 1, true) then
      text = string.format("%s|cffff8000%s|r   %s\n", text, data[1], data[2])
      
      if #data[3] > 0 then
        text = string.format("%s\n  %s\n", text, #data[3] == 1 and "Например:" or "Например:")
        for i, pair in ipairs(data[3]) do
          text = string.format("%s    |cff40bbff%s|r\n      %s\n", text, pair[1], pair[2])
        end
      end
      
      text = text .. "\n"
    end
  end
  
  self:ShowText(text == "" and ("Нет команды '"..argument.."'") or text, "Клавиатурные команды QuestHelper")
end

commands =
 {
  {"VERSION",
   "Отображает версию QuestHelper'а.", {}, QuestHelper.PrintVersion, QuestHelper},
  
  {"RECALC",
   "Сбрасывает все направления и пересчитывает их по всем доступным задачам.", {}, QuestHelper.WantPathingReset, QuestHelper},
  
  {"FTIME",
   "Переключает примерный таймер полёта.", {}, QuestHelper.ToggleFlightTimes, QuestHelper},
  
  {"PURGE",
   "Удаляет все собранные данные QuestHelper'а.", {}, QuestHelper.Purge, QuestHelper},
  
  {"HARDRESET",
   "Удаляет все собранные данные и сбрасывает настройки QuestHelper'а.", {}, QuestHelper.HardReset, QuestHelper},
   
  {"FILTER",
   "Автоматически отображает или игнорирует задачи, основываясь на критериях.",
   {{"/qh filter zone", "Переключает отображение заданий вне текущей локации."},
    {"/qh filter done", "Переключает отображение для завершённых заданий."},
    {"/qh filter level", "Переключает отображение задач, которые, вероятно, слишком сложны, учитывая ваш уровень и уровни игроков в вашей группе, а также устанавливает смещение по уровню."},
    {"/qh filter blocked", "Переключает отображение заблокированных задач, например вернуться с ещё незавершённым заданием."},
    {"/qh filter watched", "Переключает отображение только отслеживаемых заданий."}
    }, QuestHelper.Filter, QuestHelper},
  
  {"LEVEL",
   "Регулирует смещение уровня, которое использует фильтр по уровню. Естественно, фильтр по уровню должен быть включен, чтобы было возможно заметить разницу.",
   {{"/qh level", "Отображает информацию, связанную с фильтром по уровеню."},
    {"/qh level 0", "Отображение заданий выше и ниже вашего текущего уровня."},
    {"/qh level +2", "Отображение заданий также на два уровня выше вашего."},
    {"/qh level -1", "Отображение заданий только на один уровень ниже вашего."}}, QuestHelper.LevelOffset, QuestHelper},
  
  {"SCALE",
   "Регулирует размер иконок на карте, которые использует QuestHelper. Приемлимые значения от 50% до 300%.",
   {{"/qh scale 1", "Использует размер иконок по умолчанию."},
    {"/qh scale 2", "Делает иконки в два раза больше от стандартных."},
    {"/qh scale 80%", "Делает иконки немного меньше, чем их стандарнтый размер."}},
    QuestHelper.genericSetScale, QuestHelper, "scale", "размера иконок", .5, 3},
  
  {"TOOLTIP",
   "Переключает добавление информации об отслеживаемых предметах и NPC во всплывающих подсказках.",
    {}, QuestHelper.ToggleTooltip, QuestHelper},
  
  {"TRACK",
   "Переключает отображение отслеживания заданий QuestHelper'а.",
    {}, QuestHelper.ToggleTrack, QuestHelper},
  
  {"TSCALE",
   "Регулирует размер списка заданий от QuestHelper'а. Приемлимые значения от 50% до 300%.",
   {},
   QuestHelper.TrackerScale, QuestHelper},
  
  {"TLEVEL",
   "Переключает отображение уровней в списке заданий от QuestHelper'а.",
   {}, QuestHelper.ToggleTrackLevel, QuestHelper},
  
  {"TQCOL",
   "Переключает отображение окраски заданий по сложности в списке заданий от QuestHelper'а",
   {}, QuestHelper.ToggleTrackQColour, QuestHelper},
  
  {"TOCOL",
   "Переключает отображение индикации прогресса цветом в списке заданий от QuestHelper'а",
   {}, QuestHelper.ToggleTrackOColour, QuestHelper},
  
  {"TRESET",
   "Сбрасывает текущую позицию списка заданий от QuestHelper'а, если он был сдвинут за пределы экрана.",
   {}, QuestHelper.ResetTrackerPosition, QuestHelper},
  
  {"NAG",
   "Сообщает вам, есть ли у вас что-то интересное, чего нет в статичной базе данных. Проверяет задания только вашей фракции, а задания противоположной пропускаются, чтобы сэкономить память.",
     {{"/qh nag verbose", "Отображает какие-либо изменения, если они были найдены."}}, QuestHelper.Nag, QuestHelper},
  
  {"POS",
    "Отображает ваше текущую позицию. Существует, в основном, для личного удобства разработчика.",
    {}, function (qh) qh:TextOut(qh:LocationString(qh.i, qh.x, qh.y)) end, QuestHelper},
  
  {"HIDDEN",
   "Составляет список задач, скрытых QuestHelper'ом от вас. В зависимости от причины, вы также можете отобразить задачу.",
   {}, QuestHelper.ShowHidden, QuestHelper},
  
  {"FIND",
   "Поиск предмета, локации или NPC.",
   {{"/qh find item rune of teleport", "Найдёт продавцов реагентами."},
    {"/qh find npc bragok", "Найдёт распорядителя полётов в Ratchet'е."},
    {"/qh find loc stormwind 50 60", "Найдёт аукционный дом в Stormwind'е."},
    {"/qh find loc 50 50", "Найдёт центр локации, в которой вы находитесь."},
    {"/qh find something", "Ищет 'something' по всех категориях."},
    {"/qh find", "Перечисляет задачи, которые вы создали вручную, чтобы их можно было удалить."}}, QuestHelper.PerformSearch, QuestHelper},
  
  {"COMM",
   "Переключает отображение коммуникации между пользователями QuestHelper'а. Существует, в основном, для личного удобства разработчика.",
    {}, QuestHelper.ToggleComm, QuestHelper},
  
  {"RECYCLE",
   "Отображает сколько неиспользуемых объектов отслеживает QuestHelper, так что он может повторно использовать их в будущем, вместо того, чтобы создавать новые.",
    {{"/qh recycle verbose", "Отображает подробную информацию о том, какие типы таблиц наиболее распространены."},
    {"/qh recycle cache", "Отображает подробную информацию о внутреннем кэше"}}, QuestHelper.RecycleInfo, QuestHelper},
  
  {"CARTWP",
   "Переключает отображение текущей задачи с помощью аддона Cartographer Waypoints.",
    {}, QuestHelper.ToggleCartWP, QuestHelper},
  
  {"TOMTOM",
   "Переключает отображение текущей задачи с помощью аддона TomTom.",
    {}, QuestHelper.ToggleTomTomWP, QuestHelper},
  
  {"SHARE",
   "Переключает обмен целями с другими пользователями QuestHelper.",
    {}, QuestHelper.ToggleShare, QuestHelper},
  
  {"SOLO",
   "Переключает игнорирование группы. Если включено, то QuestHelper будет думать, что вы всегда одиноки. Обмен целями с группой будет отключен автоматически.",
    {}, QuestHelper.ToggleSolo, QuestHelper},
  
  {"HIDE",
   "Скрывает QuestHelper и всё, что его касается.",
    {}, QuestHelper.ToggleHide, QuestHelper},
  
  {"ANTS",
   "Переключает отображение оптимального пути на карте.",
    {}, QuestHelper.ToggleAnts, QuestHelper},
  
  {"LOCALE",
   "Выбор языка, для отображения сообщений.",
    {}, QuestHelper.SetLocale, QuestHelper},

  {"PERF",
   "Регулирует или отображает уровень производительности для маршрута. Чем выше, тем более агрессивно обновляется маршрут. Чем ниже, тем лучше производительность. Приемлимые значения от 10% до 500%.",
   {{"/qh perf", "Отображает текущий уровень производительности."},
    {"/qh perf 1", "Устанавливает уровень производительности по умолчанию."},
    {"/qh perf 50%", "Вдвое меньше фоновой обработки."},
    {"/qh perf 3", "Вычисляет маршруты в три раза агрессивней."}},
    QuestHelper.genericSetScale, QuestHelper, "perf_scale", "уровня производительности", .1, 5},
  
  {"BUTTON",
   "Переключает отображение кнопки 'QuestHelper' на карте мира.",
   {}, QuestHelper.ToggleMapButton, QuestHelper},

  {"SETTINGS",
   "Открывает меню настроек в текущем местоположении курсора мыши. Обратите внимание, что не все настройки доступны в меню.",
   {}, QuestHelper.DoSettingsMenu, QuestHelper},
  
  {"CHANGES",
   "Отображает список изменений, недавно внесённых в QuestHelper. Всегда отобржается при обнаружении обновления.",
   {}, QuestHelper.ChangeLog, QuestHelper},
  
  {"ERROR",
   "Отображает первую ошибку QuestHelper'а, сгенерированную в этом сеансе, в форме, с которой её можно будет скопировать в буфер.",
   {}, QuestHelper.ShowError, QuestHelper},
   
  {"SUBMIT",
   "Отображает инструкции, как отправить собранные данные.",
   {}, QuestHelper.Submit, QuestHelper},
  
  {"HELP",
   "Отображает список клавиатурных команд.",
   {}, QuestHelper.Help, QuestHelper}
 }

function QuestHelper_SlashCommand(input)
  local _, _, command, argument = string.find(input, "^%s*([^%s]-)%s+(.-)%s*$")
  if not command then
    command, argument = input, ""
  end
  
  command = string.upper(command)
  
  for i, data in ipairs(commands) do
    if data[1] == command then
      local st = {}
      
      for i = 5,#data do table.insert(st, data[i]) end
      table.insert(st, argument)
      
      if type(data[4]) == "function" then
        data[4](unpack(st))
      else
        QuestHelper:TextOut(data[1].." ещё не реализовано.")
      end
      
      return
    end
  end
  
  QuestHelper:Help()
end

SLASH_QuestHelper1 = "/qh"
SLASH_QuestHelper2 = "/questhelper"
SlashCmdList["QuestHelper"] = function (text) QuestHelper_SlashCommand(text) end
